# Nuxt Content Module

*Warning: Under active development. Still not stable.*

Nuxt Content grabs all content in a registered directory and converts each file into a Nuxt page. The necessary route data is received via the file's name or its front-matter.

## Installation

```
npm install nuxt-content

```
(*Please note:* `vue-content-loader` is a dev dependency, if you haven't already, please install it and save it to your `package.json`.)

Then, under `nuxt.config.js` install the module:

```
modules: [
   '@nuxtjs/content'
 ]
```

## Basic Usage

There are two main steps to using `nuxt-content`: 1) configuring how you want your content compiled and 2) creating layouts for your content pages.

### Configuration

#### Directory Options

All top level content options can be configured under the `content` property in `nuxt.config.js`.

*Note: All paths are relative to Nuxt Source Directory.*

  - `srcDir`, String that specifies the directory where the content is located. By default, all content is placed in the `/content` directory.
  - `layoutDir`, String that specifies the content's path to the content's layout directory. Defaults to `layouts/content`.
  - `route`, String that specifies the parent route, which the content will be nested under. If you'd like a top level route to be created, then make the path a slash, `/`.
  - `layout`: String that specifies the component's layout. The component can only specify the template and styles. If a script exists, it will be ignored in favor of the created script with the front-matter data.
  - `permalink`, String that specifies url path configuration options. The possible options
  are `:slug`, `:section`, `:year`, `:month`, `:day`.
  - `isPost`, Boolean that specifies whether the content requires a date. The default is true.
  - `dirs`, Array that specifies options for all content under a directory. A 2D array is also allowed to configure multiple content types. These nested configurations will override any global directory options mentioned above.

Here's an example:

```js
content: {
 // Global Options
 srcDir: "content" // default
 // Directory Options
 dirs: [
  ["posts", { // content/blog/2013-01-10-HelloWorld.md -> localhost:3000/2013/hello-world
    route: ":post",
    permalink: ':year/:slug'
  }],
  ["projects", { // content/projects/NuxtContent.md -> localhost:3000/projects/nuxt-content
    route: "projects",
     permalink: ":section/:slug",
    isPost: false
  }]
 ]
}

```

*Note: If the `route` is an existing page, the nested content will only appear if `<nuxt-child />` is present*


#### Page Options

By default, page specific data is extracted from the file name, but some options can be overridden via the front-matter of the respective file.

Front Matter Options:
  -  `slug`, String that overrides the content's url identification name.
  - `permalink`, String that overrides the content's entire url path.

For example, if you wanted to override the page's slug:

```js
// `nuxt.config.js`
content: {
  route: '/blog',
  permalink: ':year/:slug'
}

// content/2014-05-10-MyFirstPost.md -> localhost:3000/2014/1st
---
slug: "1st"
---

# Hello World!

```

### Layouts

`Nuxt-content` grabs your content's data (filename, front-matter, markdown) and renders a `script` element based on it. This allows us to inject the data into your component.

There are three important aspects to understand about the injected script element. First, your file's content are under the the `content` property. Second, since the markdown is converted to HTML, you have to include the `v-html="content"` directive in your template if you want the content to be rendered. Third, since Vue only allows one `script` element per component, you can not use your own for the content's layout, as it will be ignored in favor of the one created with your content's data.

With that out of the way, here's the fun part: you can create `templates` and `styles` to complement your component.

The templates can use interpolations (`{{ }}`) with any of the data you intend to include in your component.

Here's a basic example:

```
// layouts/content/posts.vue

<template>
<div>
  <h1> {{ title }} </h1>
  `<section v-html="content"></section>`
</div>
</template>

// content/posts/HelloWorld.vue

---
title: Hello World
--

This is my first project!

```

Will be turned into:

```
<template>
<div>
  <h1> Hello World </h1>
  <section>
    <p> This is my first project! </p>
  </section>
</div>
</template>
```

## Sites built with using `nuxt-content` module  

* [alidcastano.com](http://alidcastano.com/)
[(source)](https://github.com/alidcastano/alidcastano.github.io)


### License

MIT
