const join = require('path').join
const loaderUtils = require('loader-utils')
const loadLayout = require('./loadLayout')
const cheerio = require('cheerio')
const markdownit = require('markdown-it')
const fm = require('front-matter')

module.exports = function (source) {
  this.cacheable()

  const options = loaderUtils.getOptions(this)
  const config = options.config
  const filePath = this.resourcePath

  let $Layout = cheerio.load(loadLayout(filePath, config), {
    xmlMode: true,
    decodeEntities: true,
    lowerCaseAttributeNames: false,
    lowerCaseTags: false
  })

  return `
    ${toScript(source)}
    ${$Layout.html('template')}
    ${$Layout.html('style')}
  `
}

/**
 * Creates script element for the file's metadata.
 */
function toScript(source) {
  const parser = markdownit({
    preset: 'default',
    html: true
    // highlight: renderHighlight
  })

  const content = fm(source)
  const data = content.attributes
  delete data.layout // layout is already set
  data.content = parser.render(content.body) // md -> html

  return `
    <script>
    export default {
      data() {
        return ${JSON.stringify(data)}
      }
    }
    </script>
  `
}
