const join = require('path').join
const fs = require('fs')
const fm = require('front-matter')
const pathToRegexp = require('path-to-regexp')
const paramCase = require('param-case')
const getContentData = require('./getContentData')

/**
 * Creates top level or nested route for files in registered content directories.
 */
module.exports = function addRoutes(routes, config) {
  const contentData = getContentData(config)

  contentData.forEach(type => {
    const dir = type[0]
    const opts = type[1]
    const filesData = type[2]

    if (opts.route === "/") { // Top level route
      filesData.forEach(file => {
        routes.push(createRoute(file, opts))
      })
    } else { // Nested route
      let routeFound = false
      routes.forEach(route => {
        const isRoute = route.path.indexOf(opts.route) > -1
        if (isRoute) {
          routeFound = true
          if (!route.children) route.children = []
          filesData.forEach(file => {
            route.children.push(createRoute(file, opts))
          })
        }
      })
      if (!routeFound) throw Error(`${dir} route does not exists`)
    }
  })
}


/**
 * Gets page data via 1) front-matter 2) file name and use it to create route.
 */
function createRoute (file, options) {
  const fileSource = fs.readFileSync(file.src, 'utf-8')
  const metadata = fm(fileSource).attributes

  const fileName = file.src.replace(/^.*[\\\/]/, '')
  const fileDate = fileName.match(/!?(\d{4}-\d{2}-\d{2})/)

  const pathOpts = {
    slug: paramCase(metadata.slug || toSlug(fileName)),
    section: file.section
  }

  if (options.isPost) {
    const dateData = fileDate[0].split('-')
    pathOpts.year = dateData[0]
    pathOpts.month = dateData[1]
    pathOpts.day = dateData[2]
  }

  const toPath = pathToRegexp.compile(options.permalink)
  const permalink = toPath(pathOpts, { pretty: true })
    .replace(/%2F/gi, "/") // url encoded slash pretty

  return {
    path: join("/", permalink),
    component: file.src
  }
}

/**
 * Remove date and ext from file name.
 */
function toSlug (fileName) {
  const date = /!?(\d{4}-\d{2}-\d{2}-)/
  const ext = /(.)[^.]+$/
  return fileName.replace(date, '').replace(ext, '')
}
